/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


#include <string.h>
#include <signal.h>
#include <stdio.h>

#include <stream.h>
#include <sys/mman.h>

#ifndef __yooexcept_HXX
#include "yooexcept.hxx"
#endif
#ifndef __cluster_HXX
#include "cluster.hxx"
#endif
#ifndef __oid_HXX
#include "oid.hxx"
#endif
#ifndef __page_HXX
#include "page.hxx"
#endif
#ifndef __obj_HXX
#include "obj.hxx"
#endif
#ifndef __yoofunc_HXX
#include "yoofunc.hxx"
#endif
#ifndef __volume_HXX
#include "volume.hxx"
#endif
#ifndef __dbserver_HXX
#include "dbserver.hxx"
#endif
#ifndef __volallocator_HXX
#include "volallocator.hxx"
#endif

#include "yoo_root.hxx"
#include "dictionary.hxx"

extern void delvol(char*) ;

extern ostream& operator<<(ostream& o, const OID& oid) ;

void db_consistency(Volume* v, int repair) {

  int numObj = 0;
  int numPage = 0 ;
  int totalSize = 0 ;
  int numDelObj = 0 ;
  int numLgObj = 0 ;
  int szLgObj = 0 ;
  int pageLgObj = 0 ;
  int* objPerClass = new int[YOO_LastClassId] ;
  for (int i = 0; i < YOO_LastClassId; i++) {
    objPerClass[i] = 0 ;
  }
  
  PID pid ;
  pid.volNo = v->volid() ;
  for (i = YOO_FIRST_USED_PAGE; i < v->size(); i++) {
    pid.pageNo = i ;
    Page p ;
    v->read(&p, pid) ;
    if (p.size() > 1) {
      if (v->isused(pid, p.size()) != p.size()) {
	cout << "Trouble in free page :" << pid << endl ;
	exit (1) ;
      }
      numPage += p.size() ;
      totalSize += p.object(0)->getSlotSize() * SLOT_SIZE ;
      numObj++ ;
      pageLgObj += p.size() ;
      szLgObj += p.object(0)->getSlotSize() * SLOT_SIZE ;
      numLgObj++ ;
      i += p.size()-1 ;
    } else {
      if (p.freeSpace() == SLOT_IN_PG) {
	if (v->isused(pid, 1) == 1) {
	  if (repair) {
	    v->freepage(pid) ;
	    cout << "Fix free page :" << pid << endl ;
	  } else {
	    cout << "Trouble in free page :" << pid << endl ;
	    exit (1) ;
	  }
	} else {
	  continue ;
	}
      } else {
	numPage++ ;
	if (v->isused(pid, 1) != 1) {
	  cout << "Trouble in free page :" << pid << endl ;
	  exit(1) ;
	}
	if (p.consistency(pid,&numObj,&numDelObj,&totalSize,objPerClass)
	    ==FALSE) {
	  cout << "Inconsistent page : " << pid << endl ;
	  exit(1) ;
	}
      }
    }
  }

  cout << "Number of Pages              : " << numPage << endl ;
  cout << "Number of Long Pages         : " << pageLgObj << endl ;
  cout << "Number of objects            : " << numObj << endl ;
  cout << "Number of long object        : " << numLgObj << endl ;
  cout << "Number of deleted objects    : " << numDelObj << endl ;
  cout << "Total used size              : " << totalSize << endl ;
  cout << "Total long size              : " << szLgObj << endl ;
  if (numPage != 0) {
    cout << "Ratio per page               : " << totalSize/numPage << endl ;
  }
  cout << "Unknown class objects        : " << objPerClass[1] << endl ;
  cout << "ListNode objects             : " << objPerClass[YOO_LISTNODE_CLID]
       << endl ;
  cout << "ListLeaf objects             : " << objPerClass[YOO_LISTLEAF_CLID]
       << endl ;
  cout << "DictNode objects             : " << objPerClass[YOO_DICTNODE_CLID]
       << endl ;
  cout << "DictLeaf objects             : " << objPerClass[YOO_DICTLEAF_CLID]
       << endl ;
  for (i = 7; i < YOO_LastClassId; i++) {
    Iterator<ADM_Class> iter(YoodaDictionary->dictClass) ;
    for (; iter.more(); iter.next()) {
      if (iter.get()->clId == i) {
	cout << "class <" << iter.get()->name << "> : " <<objPerClass[i]<<endl;
      }
    }
  }
}


void option_error(char* name) 
{
  printf("usage: %s [-r] [-v] -b <basename> <volume id>\n", name);
  exit(1) ;
}
	

int main(int argc, char** argv) {

  char* base = NULL ;
  int vid = 2 ;
  int vers = 0 ;
  int repair = 0 ;
  
  for (int i = 1; i < argc; i++) {
    if (strcmp(argv[i], "-b") == 0) {
      base = argv[i+1] ;
      i++ ;
    } else if (strcmp(argv[i], "-v") == 0) {
      vers = 1 ;
    } else if (strcmp(argv[i], "-r") == 0) {
      repair = 1 ;
    } else {
      vid = atoi(argv[i]) ;
    }
  }

  if (base == NULL) {
    base = getenv("YOODA_BASE") ;
    cerr << "No database set" << endl ;
    if (base == NULL) option_error(argv[0]) ;
  }

  if (vers) {
    printf("YOODA version 1.2 <%s>, Copyright (C) 1994 Eric Abecassis
YOODA comes with ABSOLUTELY NO WARRANTY; for details type `COPYING'
in YOODA_HOME directory. This is free software, and you are welcome to 
redistribute it under certain conditions; for details read `COPYING'
or 'COPYING.LIB' in YOODA_HOME directory\n", argv[0]) ;
  }

  yoo_connect(base, MONOUSER) ;

  YOO_LastClassId = YoodaDictionary->lastClass ;
  YOO_STAT = TRUE ;
  Volume* vol = YOO_DBServer->volume(vid) ;
  if (vol == NULL) {
    cerr << "Volume does not exist" << endl ;
  } else {  
    db_consistency(vol, repair) ;
  }

  yoo_disconnect() ;
}
